# == Schema Information
#
# Table name: forums
#
#  id                :integer          not null, primary key
#  user_id           :integer
#  forum_topic_id    :integer
#  owner_name        :string(50)       default("")
#  parent            :integer          default(0)
#  state             :string(50)       default("new")
#  tag               :integer          default(-1)
#  hold              :boolean          default(FALSE)
#  content           :text
#  closed            :boolean          default(FALSE)
#  response          :string(255)      default("none")
#  restricted        :boolean          default(FALSE)
#  needs_translating :boolean          default(FALSE)
#  has_photo         :boolean          default(FALSE)
#  resolution        :text
#  created_at        :datetime
#  updated_at        :datetime
#

# noinspection RubyQuotedStringsInspection
class Forum < ActiveRecord::Base
  include  ForumSharedCode
  require 'json'
  belongs_to :forum_topic
  has_many :forum_translations, dependent: :destroy
  has_many :forum_photos, dependent: :destroy
  has_many :forum_people_lists, dependent: :destroy
  has_many :forum_hyperlinks, dependent: :destroy

  #todo add hyperlinks in comments

  USER_BLOCK_SIZE= 300

  def self.do_admin_action(params,user_id)
    params[:users1]=[] if params[:users1].nil?
    params[:users2]=[] if params[:users2].nil?
    user_ids=(params[:users1] + params[:users2])
    user_ids.uniq!
    (0...user_ids.size).each{|i| user_ids[i]=user_ids[i].to_i}
    user_ids.delete(0)
    case params[:forum_action]
      when 'suspend posting'
        users_ids.each{|user| User.where(id:user).update_all(forum_posting_priviages:false)}
      when 'suspend all'
        users_ids.each{|user| User.where(id:user).update_all(forum_reading_priviages:false)}
        users_ids.each{|user| User.where(id:user).update_all(forum_posting_priviages:false)}
      when 'send email to'
        UserMailer.note_to_forum_users(params,user_ids).deliver
      when 'create distribution' 'add to distribution' ,'remove from distro'
        ForumPeopleList.manage_list(params,user_ids)
      when 'add topic monitor'
        ForumTopic.add_topic_monitor(params,user_ids)
      else
        # type code here
    end if User.forum_admin?(user_id)
    {action:'nothing to do'}
  end

  def self.senate(params,user_id)
    if params[:topic]== "select topic"
      {to_do:'nothing'}
    else
      admin= User.forum_admin?(user_id)
      topic_id=ForumTopic.where(name:params[:topic]).pluck(:id)[0]
      index=Forum.clear_user_topic(user_id,topic_id)
      the_list=[]
      plucking=       [:id, :parent, :content, :user_id,:owner_name,:state,:restricted,:tag,:has_photo]
      plucking_names= [:id, :parent, :content,:owner,:owner_name,:comment_state,:restricted,:tag,:has_photo,:needs_translation,:empty_list] #we're going to append empty list to the reply
      moderator=ForumPeopleList.check_people({topic:topic_id,user_id:user_id,list:'moderator'}) || admin
      if params[:review]=='true' && moderator
        test=Forum.where(forum_topic_id:topic_id,state:'new').pluck(:id)
        unless test.empty?
          item1=ForumAssist.named_array(ForumTranslation.get_localized_forum(ForumLanguage.get_me(params),{lookup:{forum_topic_id:topic_id},plucking:plucking,size:0}),plucking_names, d2=false)
          item1.store(:moderator,true)
          item1.store(:review,true)
          new_list=  ForumAssist.named_array( ForumTranslation.get_localized_forum(ForumLanguage.get_me(params),{lookup:{forum_topic_id:topic_id,state:'new'},ordering:([:topic]),plucking:plucking,size:'all'}),plucking_names , d2=true)
          new_list.unshift(item1)
          the_list=new_list
        end
      else
        senate_list=ForumAssist.named_array(ForumTranslation.get_localized_forum(ForumLanguage.get_me(params),{lookup:{forum_topic_id:topic_id,hold:false},ordering:[:parent,:created_at],plucking:plucking,size:'all'}), plucking_names , d2=true)
        if senate_list.empty?
          []
        else
          voting=ForumAssist.named_array(ForumTopic.where(id:topic_id).pluck(:vote,:votes)[0],[:vote,:votes],d2=false)
          voting[:vote] ? votes=ForumTopic.count_votes(voting[:votes].clone) : votes=0
          parent=Forum.look_for_key(senate_list,:parent,0)
          new_list=[]
          subscribed=ForumPeopleList.check_people({topic:topic_id,user_id:user_id,list:'subscribers'})
          parent=set_new_senate_entry(senate_list,new_list,parent,user_id, {moderator:moderator,vote:voting[:vote],votes:votes,subscribed:subscribed}) unless parent.nil?
          Forum.is_child_there?(senate_list,new_list,0,parent,user_id)        #push first child
          new_list.delete_if  {|record|
            test=ForumPeopleList.check_people_list({topic:record[:id], user_id:user_id,list:'invitation'}) if (record[:restricted] && !record[:owner])
            if test.nil?
              case record[:state]
                when 'reject' then  test= false
                when 'new' then  test= !(ENV['FORUM_NEW_RULE']=='no')
                else  test=true
              end
            end
            !test
          }
          the_list=new_list
        end
      end
      {the_list:the_list,index:index}
    end
  end

  def self.clear_user_topic(user_id,topic_id)
    comments=ForumAssist.named_array(Forum.where(forum_topic_id:topic_id).order(:tag).pluck(:id,:tag),[:id,:tag],d2=true)
    tags=[]
    test_comments=comments.clone
    test_comments.each{ |comment|
      if (comment[:tag].to_i) < USER_BLOCK_SIZE*user_id && comment[:tag].to_i <= USER_BLOCK_SIZE*(user_id+1)
          comments.delete_at(comments.index(comment))
      else
        tags.push((comment[:tag].to_i)%USER_BLOCK_SIZE )
        comment[:tag]= tags.last
      end
    }
    if tags.empty?
      index=0
    elsif tags.index(USER_BLOCK_SIZE*1/3).nil? && tags.index(USER_BLOCK_SIZE*2/3).nil?
      index=USER_BLOCK_SIZE*1/3
    elsif tags.index(USER_BLOCK_SIZE*2/3).nil?
      index= USER_BLOCK_SIZE*2/3
      (0...tags.size).each {|i|
        Forum.where(id:comments[i][:id]).update_all(tag:-1) if tags[i] < USER_BLOCK_SIZE*1/3
      }
    elsif tags.index(USER_BLOCK_SIZE*0/3).nil?
      index=0
      (0...tags.size).each {|i|
        Forum.where(id:comments[i][:id]).update_all(tag:-1) if tags[i] >= USER_BLOCK_SIZE*1/3 && tags[i]< USER_BLOCK_SIZE*2/3
      }
    else tags.index(USER_BLOCK_SIZE*1/3).nil?
      index=USER_BLOCK_SIZE*1/3
      (0...tags.size).each {|i|
        Forum.where(id:comments[i][:id]).update_all(tag:-1) if tags[i] >= USER_BLOCK_SIZE*2/3
      }
    end
    index+user_id*USER_BLOCK_SIZE
  end

  def self.set_new_senate_entry(senate_list,new_list,index,user_id,options)
    senate_list[index].store(:photos,ForumPhoto.get_photos(senate_list[index][:id])) if senate_list[index][:has_photo]
    senate_list[index].store(:hyperlinks,ForumHyperlink.get_links(senate_list[index][:id]))
    senate_list[index].store(:indent,Forum.get_indent(senate_list[index][:id],0) )
    senate_list[index][:owner]= senate_list[index][:owner]==user_id
    senate_list[index].store(:moderator,options[:moderator]) unless options[:moderator].nil?
    senate_list[index].store(:vote,options[:vote])  unless options[:vote].nil?
    senate_list[index].store(:votes,options[:votes]) unless options[:votes].nil?
    senate_list[index].store(:admin,options[:admin]) unless options[:admin].nil?
    senate_list[index].store(:subscribed,options[:subscribed]) unless options[:subscribed].nil?

    new_list.push(senate_list[index])
    id=senate_list[index][:id]
    senate_list.delete_at(index)
    id
  end

  def self.get_indent(id,index)
    parent=Forum.where(id:id).pluck(:parent)[0]
    index=Forum.get_indent(parent,index+1) unless parent==0 || parent.nil?
    index
  end

  def self.look_for_key(list,key,test)
    index_out=nil
    (list.size-1).downto(0).each { |index|
      index_out=index if list[index][key]==test
    } unless list.empty?
    index_out
  end

  def self.owner(user_id,id)
    Forum.where(id:id).pluck(:user_id)[0]==user_id
  end

  def self.add_comment(params,user_id)
    topic_id=ForumTopic.where(name:params[:topic]).pluck(:id)[0]
    unless params[:comment].nil? || params[:comment].empty? || !User.where(id:user_id).pluck(:forum_posting_privileges)[0]
      has_photo=params[:already_has_photo] == 'true' || params[:needs_photo] == 'true'
      params[:id]=Forum.where(user_id:user_id,tag:params[:tag].to_s).pluck(:id)[0]  if params[:id].nil?
      if params[:edit]=='true' && (user_id==Forum.where(id:params[:id]).pluck(:user_id)[0] ||   User.forum_admin?(user_id))
        forum_id=params[:id]
        Forum.where(id: params[:id]).update_all(content: params[:comment],has_photo:has_photo, parent: params[:parent],restricted:params[:restricted])
      else
        owner_name=User.where(id:user_id).pluck(:forum_post_name)[0]
        forum_id= Forum.create(parent:params[:parent],forum_topic_id:topic_id,tag:params[:tag], has_photo:has_photo, user_id:user_id,content:params[:comment],restricted:params[:restricted],owner_name:owner_name).id
      end
      ForumTranslation.add_one(params,topic_id,user_id,forum_id) if params[:translating] && User.forum_translating_privileges?(user_id)
      ForumPeopleList.make_invitation(params,forum_id,user_id) if params[:restricted]=='true'
      ForumPeopleList.add_to_invitation(params,forum_id,user_id) unless params[:invitees].nil?
      ForumHyperlink.add_set(params,forum_id) unless params[:link_ref].nil?
    end
    {parent:params[:parent],id:forum_id,topic_id:topic_id,tag:params[:tag],edit:params[:edit]=='true'}
  end

  def self.get_forum_actions(params,user_id)
    alist= []
    alist.push(['select action',I18n.t('forum.select_action')])
    if User.forum_admin?(user_id)
      alist.push(['suspend posting',I18n.t('forum.suspend_posting')])
      alist.push(['suspend all',I18n.t('forum.suspend_all')])
      alist.push(['send email to',I18n.t('forum.send_email_to')])
      alist.push(['create distribution',I18n.t('forum.create_distribution')])
      alist.push(['add to distribution',I18n.t('forum.add_to_distribution') ])
      alist.push(['add topic monitor',I18n.t('forum.Add_Topic_Monitor') ])

    end
    {list_data:ForumAssist.named_array(alist,[:item,:value],d2=true),id:'forum-actions'}
  end

  def self.change_state(params,user_id)
    Forum.where(id:params[:id]).update_all(state: params[:function],resolution:params[:resolution]) if ForumPeopleList.check_people({topic:params[:id], user_id:user_id,list:'moderator'})  ||  User.forum_admin?(user_id)
    {action:'nothing to do'}
  end

  def self.set_the_rest(params,user_id)
    topic_id=ForumTopic.where(name:params[:topic]).pluck(:id)[0]
    Forum.where(forum_topic_id:topic_id, state: 'new').update_all(state: params[:state]) if ForumPeopleList.check_people({topic:topic_id,user_id:user_id,list:'moderator'}) ||  User.forum_admin?(user_id)
    {action:'nothing to do'}
  end

  def self.is_child_there?(senate_list,new_list, parent,me,user_id)
    child=Forum.look_for_key(senate_list,:parent,me)
    unless child.nil?
      child=Forum.set_new_senate_entry(senate_list, new_list, child, user_id, {})
      is_child_there?(senate_list, new_list, me, child, user_id) unless senate_list.empty?
    end
    if me!=parent && me!=0
      Forum.is_child_there?(senate_list, new_list,parent ,parent, user_id) unless senate_list.empty?
    end
  end

  rails_admin do
    list do
      field :id
      field :user_id
      field :forum_topic_id
      field :restricted
      field :parent
      field :state
    end
  end
end
