# == Schema Information
#
# Table name: forum_languages
#
#  id         :integer          not null, primary key
#  user_id    :integer
#  code       :string(5)
#  name       :string(50)
#  local_name :string(50)
#  region     :string(50)
#  suggestion :boolean
#  state      :string(50)       default("suggesiton")
#  active     :boolean          default(FALSE)
#  created_at :datetime
#  updated_at :datetime
#

class ForumLanguage < ActiveRecord::Base
  include  ForumSharedCode
  require 'json'

  has_many :forum_translations, dependent: :destroy

  def self.toggle_language(params,user_id)  #this has to be an all new  language
    state= ForumLanguage.where(name:params[:language]).pluck(:active)[0]
    ForumLanguage.where(name:params[:language]).update_all(active:!state) if User.forum_admin?(user_id)
    {action:'nothing to do'}
  end

  def self.add_language(params,user_id)
    if User.forum_admin?(user_id)
       ForumLanguage.create(code:params[:code],name:params[:name],local_name:params[:local])
    end
  end

  def self.languages(params)
    ForumLanguage.init_language_set if ForumLanguage.first.nil?
    params[:active] == 'true' ?
      alist=ForumLanguage.where(active:true).order(:code).pluck(:id,:name,:code,:local_name,:active)  :
      alist=ForumLanguage.where(active:false).order(:code).pluck(:id,:name,:code,:local_name,:active)
      unless alist.empty?
        alist=ForumAssist.named_array(alist,[:id,:item,:value,:local_name,:state],d2=true)
        alist=ForumTranslation.translate_list(alist,'','language')
      end
      alist.unshift({value: I18n.t('forum.select_language'),item: I18n.t('forum.select_language') ,state:'open',id:0})
      {list_data:alist,id:params[:list]}
  end

  def self.get_suggestions(params,user_id)
    if User.forum_admin?(user_id)
      data= ForumLanguage.where(suggestion:true).pluck(:content, :user_id, :id)
      data=ForumAssist.named_array(data, [:suggestion, :user,:id ], d2=true) unless data.empty?
      data=Forum.add_user_name(data.clone) unless data.empty?
      {data:data,list:'topic-prososals-list'}
    else
      {reply:'nothing doing'}
    end
  end

  def self.suggestion(params,user_id)  #add new category or topic
    ForumLanguage.create(suggestion:true,content:params[:suggestion],user_id:user_id,hold:true)
    {response:'nothing to do'}
  end

  def self.suggestion_close(params,user_id)
    ForumLanguage.where(id:params[:id]).update_all(state:'closed') if User.forum_admin?(user_id)
  end

  def self.get_me(params)
    params[:language] = params[:language][0] if params[:language].is_a?(Array)
    code=params[:language].split('-')[0].strip
    ForumLanguage.where(code:code).pluck(:id)[0]
  end

  def self.init_language_set

    language_table = %w(af.Afrikaans.Afrikaans ar.Arabic.العربية be.Belarusian.беларускі bg.Bulgarian.български ca.Catalan.Català cs.Czech.čeština da.Danish.dansk de.German.dansk el.Greek.ελληνικά en.English.English es.Spanish.español et.Estonian.eesti eu.Basque.Euskal fa.Farsi.فارسی fi.Finnish.suomalainen fo.Faroese.føroyskt fr.French.français ga.Irish.Gaeilge gd.Gaelic.gaélico he.Hebrew.עברית hi.Hindi.हिंदी hr.Croatian.hrvatski hu.Hungarian.magyar id.Indonesian.Indonesia is.Icelandic.Icelandic it.Italian.italiano ja.Japanese.日本の ji.Yiddish.ייִדיש ko.Korean.한국의 lt.Lithuanian.Lietuvos lv.Latvian.Latvijas mk.Macedonian.македонски ms.Malaysian.Malaysia mt.Maltese.Malti nl.Dutch.Nederlands no.Norwegian.norsk pl.Polish.polski pt.Portuguese.português ro.Romanian.român ru.Russian.русский sk.Slovak.slovenský sl.Slovenian.slovenščina sq.Albanian.shqiptar sr.Serbian.српски sv.Swedish.Svenska th.Thai.ภาษาไทย tr.Turkish.Türk uk.Ukrainian.Український ur.Urdu.اردو vi.Vietnamese.Việt zh.Chinese.中國的 zu.Zulu.Zulu)

    language_table.each {|entry|
      set=ForumAssist.named_array(entry.split('.'),[:code,:language,:local],d2=false)
      ForumLanguage.create(code:set[:code], name:set[:language],local_name:set[:local],active:false  )
    }

  end

end
