# == Schema Information
#
# Table name: forum_people_lists
#
#  id         :integer          not null, primary key
#  owner_id   :integer
#  name       :string(20)
#  people     :binary           default("--- []\n")
#  list       :string(20)
#  created_at :datetime
#  updated_at :datetime
#

class ForumPeopleList < ActiveRecord::Base
  include ForumSharedCode
  require 'json'

  belongs_to :forum_topic
  belongs_to :forum_category
  belongs_to :forum

  serialize :people

  def self.get_people_list(user_id, list)
    people = []
    people.push(user_id)
    unless list.nil?
      list.gsub(',', ';')
      new_list=list.split(';')
      email_list=[]
      user_list=[]
      new_list.each { |entry|
        if entry.include?('@')
          email_list.push(entry)
        else
          user_list.push(entry)
        end
      }
      user_list.each { |entry|
        id=User.where(forum_post_name: entry).pluck(:id)[0]
        people.push id unless id.nil?
      }
      email_list.each { |entry|
        id=User.where(email: entry).pluck(:id)[0]
        people.push id unless id.nil?
      }
    end
    people
  end

  def self.manage_list(params, user_ids)
    case params[:forum_action]
      when 'create distribution'
        ForumPeopleList.create(name: params[:new_list_name], people: user_ids.to_json, list: 'distribution list')
      when 'add to distribution'
        alist=user_ids+JSON.load(ForumPeopleList.where(id: params[:distribution_list]).pluck(:people)[0])
        alist.uniq!
        ForumPeopleList.where(id: params[:distribution_list]).update_all(people: alist.to_json)
      when 'remove from distro'
        alist=JSON.load(ForumPeopleList.where(id: params[:distribution_list]).pluck(:people)[0])-user_ids
        ForumPeopleList.where(id: params[:distribution_list]).update_all(people: alist.to_json)
      else
    end
  end

  def self.get_forum_people_list(params)
    which=ForumPeopleList.get_which_owner(params)
    which.store(:list,params[:list])
    people= JSON.load(ForumPeopleList.where(which).pluck(:people)[0])
    if people.nil?
      ForumPeopleList.create(owner_id: id, list: alist)
      []
    else
      people
    end
  end

  def self.get_which_owner(params)
    if  !params[:topic].nil?
       {forum_topic_id:params[:topic]}
    elsif   !params[:forum].nil?
      {forum_id:params[:topic]}
    elsif  !params[:category].nil?
      {forum_category_id:params[:topic]}
    else
      {}
    end
  end

  def self.get_forum_emails(params, user_id)
    User.forum_admin?(user_id) ? alist=User.all.pluck(:email, :id) : alist= []
    alist.unshift([I18n.t('forum.select_none'), 'select none', 0])
    {list_data: ForumAssist.named_array(alist, [:value, :item], d2=true), name: 'forum-user-email'}
  end

  def self.get_forum_users(params, user_id)
    User.forum_admin?(user_id) ? alist = User.all.pluck(:forum_post_name, :id) : alist =[]
    alist.unshift([I18n.t('forum.select_none'), 'select none', 0])
    {list_data: ForumAssist.named_array(alist, [:value, :item], d2=true), name: 'forum-user-tag'}
  end

  def self.check_restrictions(alist, user_id)
    blist=alist.clone
    blist.each { |record|
      alist.delete(record) if Forum.check_user_invite(record[:id], user_id) if record[:restricted]
    }
    alist
  end

  def self.get_forum_distribution_lists(user_id)
    User.forum_admin?(user_id) ? alist=ForumPeopleList.where(list: 'distribution list').pluck(:name, :list, :id) : alist=[]
    alist.unshift([I18n.t('forum.select_list'), 'select list', 0])
    {list_data: ForumAssist.named_array(alist, [:value, :item, :id], d2=true), id: 'forum-distribution-lists'}
  end

  def self.is_a_moderator?(user_id)
    the_lists=ForumPeopleList.where(list: 'moderator').pluck(:people)
    moderators=[]
    the_lists.each { |a_list|
      moderators+= a_list
      moderators.uniq!
    }
    moderators.include?(user_id)
  end

  def self.make_invitation(params, forum_id, user_id)
    invitees=ForumPeopleList.get_people_list(user_id, params[:invitees])
    peoplelist=ForumPeopleList.where(list: 'invitation', owner_id: forum_id).first_or_create { |invitation|
      invitation.owner_id=forum_id
      invitation.list='invitaton'
    }
    peoplelist.invite_these_guys(invitees)
  end

  def self.add_to_list(params, users)
    if params[:name].nil?
      a_list=ForumPeopleList.where(list: params[:list], owner_id: params[:owner]).first_or_create { |new|
        new.list=params[:list]
        new.owner_id=params[:owner]
      }
    else
      a_list=ForumPeopleList.where(list: 'distribution list', name: params[:list]).first_or_create { |new|
        new.list = 'distribution list'
        new.name = params[:list]
      }
    end
    a_list.invite_these_guys(users)
  end

  def self.check_people(params) # owner_id,user_id,list)
    which=ForumPeopleList.get_which_owner(params)
    which.store(:list,params[:llist])
    people=ForumPeopleList.where(which).pluck(:people)[0]
    (people.nil? || people.empty?) ? false : people.include?(params[:user_id].to_s)
  end

  def self.add_to_invitation(params, id, user_id)
    peoplelist = ForumPeopleList.find_people_list(params, id)
    peoplelist.add_these_guys(ForumPeopleList.get_people_list(user_id, params[:invitees])) unless peoplelist.nil?
  end

  def self.find_people_list(params, forum_id)
    which=ForumPeopleList.get_which_owner(params)
    which.store(:list,'invitation')
    list = ForumPeopleList.find_by(which)
    if list.nil?
      parent=Forum.where(id: forum_id).pluck(:parent)[0]
      if parent == 0
        topic=Forum.where(id:forum_id).pluck(:forum_topic_id)
        ForumPeopleList.find_by(forum_topic_id:topic,list:params[:list])
      else
        ForumPeopleList.find_people_list(params, parent) unless parent.nil?
      end
    end
  end

  def invite_these_guys(invitees)
    invitees += JSON.load(self.people) unless  JSON.load(self.people).nil?
    invitees.uniq!
    self.people = invitees.to_json
    self.save
  end
end
