# == Schema Information
#
# Table name: forum_topics
#
#  id                :integer          not null, primary key
#  forum_category_id :integer
#  user_id           :integer
#  name              :string(50)
#  state             :string(20)
#  closed            :boolean          default(FALSE)
#  suggestion        :boolean
#  votes             :binary           default("--- []\n")
#  vote              :boolean          default(FALSE)
#  restricted        :boolean          default(FALSE)
#  created_at        :datetime
#  updated_at        :datetime
#

class ForumTopic < ActiveRecord::Base
  include  ForumSharedCode
  require 'json'
  belongs_to :forum_category
  has_many :forums, dependent: :destroy
  has_many :forum_people_lists, dependent: :destroy
  serialize :votes

  def self.topics(params,user_id)
    if ForumTopic.first.nil?
      ForumTopic.create(name:'select topic')
      ForumTopic.create(name:'request new topic')
    end
    unless  params[:category] == 'select category'
      category_id=ForumCategory.where(name:params[:category]).pluck(:id)[0]
      params[:hold_test]=='true' ?
          topics=ForumAssist.named_array(ForumTopic.where(forum_category_id:category_id,closed:false).pluck(:id,:name,:state,:restricted,:closed),  [:id,:item,:state,:restricted,:closed],d2=true)  :
          topics=ForumAssist.named_array(ForumTopic.where(forum_category_id:category_id).pluck(:id,:name,:state,:restricted,:closed),  [:id,:item,:state,:restricted,:closed],d2=true)
      topics=[] if topics[0][:id].nil?
      topics=ForumPeopleList.check_restrictions(topics,user_id) if params[:restricted]=='true'
      a=ForumAssist.named_array(ForumTopic.where(name:'select topic').pluck(:id,:name,:state,:restricted,:closed)[0],   [:id,:item,:state,:restricted,:closed],d2=false)
      b=ForumAssist.named_array(ForumTopic.where(name:'request new topic').pluck(:id,:name,:state,:restricted,:closed)[0], [:id,:item,:state,:restricted,:closed],d2=false)
      topics.unshift(a)
      topics.push(b)
      {list_data:ForumTranslation.translate_list(topics,params[:language],'topic'),id:params[:list]}
    end
  end

  def self.voting_booth?(params)
    topic=ForumTopic.find_by(name:params[:topic])
    topic.vote=!topic.vote
    topic.save
    {vote:topic.vote,votes:'0'}
  end

  def self.add_topic_monitor(params,user_ids)
    topic=ForumTopic.where(name:params[:topic]).pluck(:id)[0]
    ForumPeopleList.add_to_list({list:'monitor',owner:topic},user_ids)
  end

  def self.get_suggestion(params,user_id)
    if User.forum_admin?(user_id)
      data=ForumTopic.where(suggestion:true).pluck(:content, :user_id, :forum_category_id,:id,:owner_name)
      data=ForumAssist.named_array(data, [:suggestion, :user, :category, :id, :owner_name], d2=true) unless data.empty?
      data=ForumAssist.add_user_name(data.clone) unless data.empty?
      {data:data,list:'topic-prososals-list'}
    else
      {response:'nothing doing'}
    end
  end

  def self.suggestion(params,user_id)  #add new category or topic
    ForumTopic.create(suggestion:true,content:params[:suggestion],user_id:user_id)
    {response:'nothing to do'}
  end

  def self.suggestion_close(params,user_id)
    ForumTopic.where(id:params[:id]).update_all(state:'closed') if User.forum_admin?(user_id)
    {response:'nothing to do'}
  end

  def self.count_votes(votes)
    count=votes.length
    count2=votes.delete_if{|element| element=='-1'
    }.length
    2*count2-count
  end

  def self.cast_vote(topic,user_id,vote)
    topic_record=ForumTopic.find_by(name:topic)
    voters=ForumPeopleList.get_forum_people_list({topic:topic_record.id,list:'voters'})
    votes=JSON.load(topic_record.votes)
    voters.empty? ? index=nil : index=voters.index(user_id)
    if vote == '1' || vote == '-1'
      if index.nil?
        voters.push(user_id)
        votes.push(vote)
      else
        votes[index]=vote
      end
    end
    ForumPeopleList.where(owner_id:topic_record.id,list:'voters').update_all(people:voters.to_json)
    topic_record.votes=votes.to_json
    topic_record.save
    votes.empty? ? 0 : ForumTopic.count_votes(votes.clone)
  end

  def self.subscribe(topic,user_id) #if they're in the list unsubscribe else subscribe
    id=ForumTopic.find_by(name:topic).pluck(:id)[0]
    subscribers=Json.load(ForumPeopleList.where(id:id,list:'subscribers').pluck(people)[0])
    index= subscribers.index(user_id.to_s) unless subscribers.empty?
    index.nil? ? subscribers.push(user_id.to_s) : subscribers.delete_at(index)
    ForumPeopleList.where(id:id,list:'subscribers').update_all(people:subscribers.to_json)[0]
  end

  def self.add_topic(params,user_id)
    params[:user_id]=user_id if params[:user_id] == 'me'
    category=ForumCategory.where(name:params[:category]).pluck(:id)[0]
    User.forum_admin?(user_id) ?
      id=ForumTopic.create(name:params[:topic],user_id:params[:user_id],forum_category_id:category).id  :
      id='not happening'
    {id:id}
  end

  def self.edit_topic(params,user_id)
    Forum.where(id: params[:id]).update_all(name:params[:newline]) if User.forum_admin?(user_id)
  end

  def self.remove_topic(params,user_id)
    case(params[:state_action])
      when 'del'  then Forum.where(id:params[:id]).update_all(closed:true)
      when 'open' then Forum.where(id:params[:id]).update_all(closed:false)
      else
        # type code here
    end  if User.forum_admin?(user_id)
    {action:'nothing to do'}
  end

end
