# == Schema Information
#
# Table name: forum_translations
#
#  id                :integer          not null, primary key
#  forum_id          :integer
#  forum_topic_id    :integer
#  forum_category_id :integer
#  forum_language_id :integer
#  user_id           :integer
#  suggestion        :boolean          default(FALSE)
#  state             :string(20)       default("new")
#  content           :text
#  resolution        :text
#  created_at        :datetime
#  updated_at        :datetime
#

class ForumTranslation < ActiveRecord::Base
  include  ForumSharedCode
  require 'json'
  belongs_to :forum
  belongs_to :forum_category
  belongs_to :forum_topic
  belongs_to :forum_hyperlink

  def self.translate_to(list)
    list.each{|entry|
      translations=ForumAssist.named_array(ForumTranslation.where(forum_id:entry[:id]).pluck(:language,:content),[:language,:content],d2=true)
      entry.store(:translations,translations)
    }
    list
  end

  def self.get_localized_forum(language,params)
    params={language:0} if params.nil?  #won't match so will default to non-translated copy
    plucking=ForumAssist.make_string_list(params[:plucking])
    ordering=ForumAssist.make_string_list(params[:ordering]) unless params[:ordering].nil?
    params[:ordering].nil? ?
      alist=Forum.where(params[:lookup]).pluck(plucking) :
      alist= Forum.where(params[:lookup]).order(ordering).pluck(plucking)
    unless alist.empty?
      indexes=plucking.split(',')
      content=indexes.index('content')
      language_list=ForumLanguage.where(active:true).order(:code).pluck(:id,:name)
      language_list=ForumAssist.named_array(language_list,[:id,:item],d2=true)
      if alist.is_a?(Array)
        alist.each { |a_entry|
          entry=ForumAssist.named_array(a_entry,params[:plucking],d2=false)
          translation = ForumTranslation.where(forum_id:entry[:id],forum_language_id:language).pluck(:content)[0] unless language.nil?
          a_entry[content]=translation  unless translation.nil?
          full_list=ForumTranslation.where(forum_id:entry[:id]).pluck(:forum_language_id)
          a_entry.push(!(full_list.size==language_list.size))
          if language_list.empty?
            a_entry.push(['nothing'])
          else
            this_list=[]
            language_list.each{ |thisone| this_list.push(thisone[:item]) unless full_list.include?(thisone[:id]) }
            a_entry.push(this_list.clone.to_json )
          end
        }
      else
        translation = ForumTranslation.where(forum_id:alist[:id],forum_language_id:language).pluck(:content)[0]
        alist[:content]=translation  unless translation.nil?
      end
      params[:size] == 0   ?  alist[0]  : alist
    end
  end

  # noinspection RubyLiteralArrayInspection
  def self.get_translations(params)
    language = ForumLanguage.get_me(params)
    topic = ForumTopic.where(name:params[:topic]).pluck(:id)[0]
    ForumAssist.named_array(ForumTranslation.where(forum_topic_id:topic,forum_language_id:language,state:['new','open']).pluck(:forum_id,:content,:suggestion),[:forum_id,:translation,:suggestion],d2=true)
  end

  def self.add_one(params,topic_id,user_id,forum_id)
    unless params[:translation] == ''
      if ForumPeopleList.check_people({topic:topic_id,list:'moderator',user_id:user_id})  && !ForumTranslation.where(forum_language_id:ForumLanguage.get_me(params),forum_id:params[:id]).nil?
        ForumTranslation.where(forum_id:forum_id,forum_topic_id:topic_id, forum_language_id:ForumLanguage.get_me(params)).update_all(content:params[:translation],suggestion:false,state:'open')
        ForumTranslation.where(forum_id:params[:id],forum_topic_id:topic_id,suggestion:true).update_all(state:'closed')
      else
        suggestion = !ForumTranslation.where(forum_language_id:ForumLanguage.get_me(params),forum_id:forum_id).nil?
        ForumTranslation.create(forum_id:forum_id,user_id:user_id,forum_topic_id:topic_id, forum_language_id:ForumLanguage.get_me(params), content:params[:translation],suggestion:suggestion,state:'new')
      end
    end
  end

  def self.translate_list(alist,language,list_name)
    alist.each.each {|entry|
      translation=ForumTranslation.find_by(forum_id:entry[:id],forum_language_id:language)  unless language.nil?
      case list_name
        when 'category'
          entry.store(:item,ForumCategory.where(id:entry[:id]).pluck(:name)[0] )
          entry.store(:value,entry[:item])
        when 'topic'
          entry.store(:item,ForumTopic.where(id:entry[:id]).pluck(:name)[0] )
          entry.store(:value,entry[:item])
        when 'language'
          translation ={content:entry[:value].concat(' - ').concat(entry[:local_name])}
          entry.store(:value,'wait')
        else
      end
      entry[:value]=translation[:content] unless translation.nil?
    }
    alist
  end

end
