class AddForums < ActiveRecord::Migration
  def change
#_____________________________________forum  users______________________________________________________#

    add_column  :users,  :forum_posting_privileges,    :boolean, default:true
    add_column  :users,  :forum_reading_privileges,    :boolean, default:true
    add_column  :users,  :forum_admin,                 :boolean, default:false
    add_column  :users,  :forum_translating_privileges,:boolean, default:true
    add_column  :users,  :forum_post_name,             :string

    add_index :users, :forum_post_name, name: :index_users_on_forum_post_name
    add_index :users, :forum_posting_privileges, name: :index_users_on_forum_posting_privileges
    add_index :users, :forum_reading_privileges, name: :index_users_on_forum_reading_privileges
    add_index :users, :forum_translating_privileges, name: :index_users_on_forum_translating_privileges
    add_index :users, :forum_admin, name: :index_users_on_forum_admin
#_______________________________________________________________________________________________________#

    create_table :forums do |t|
      t.belongs_to  :user
      t.belongs_to  :forum_topic
      t.string      :owner_name,    limit:50,     default:''
      t.integer     :parent,                      default:0
      t.string      :state,         limit:50,     default:'new'
      t.integer     :tag,                         default:-1
      t.boolean     :hold,                        default:false
      t.text        :content
      t.boolean     :closed,                      default:false
      t.string      :response,                    default:'none'
      t.boolean     :restricted,                  default:false
      t.boolean     :needs_translating,           default:false
      t.boolean     :has_photo,                   default:false
      t.text        :resolution
      t.timestamps
    end

    add_index :forums, :forum_topic_id,                 name: :index_forums_on_forum_topic_id
    add_index :forums, :user_id,                        name: :index_forums_on_user_id
    add_index :forums, %W(user_id forum_topic_id),      name: :index_forums_on_user_id_and_forum_topic_id
    add_index :forums, %W(forum_topic_id state),        name: :index_forums_on_topic_and_state
    add_index :forums, :closed,                         name: :index_forums_on_closed
    add_index :forums, :hold,                           name: :index_forums_on_hold
    add_index :forums, :parent,                         name: :index_forums_on_parent
    add_index :forums, :state,                          name: :index_forums_on_state
    add_index :forums, :restricted,                     name: :index_forums_on_restricted
    add_index :forums, :has_photo,                      name: :index_forums_on_has_photo

    create_table :forum_hyperlinks do |t|
      t.belongs_to :forum
      t.string  :link_address
      t.string  :link_text
      t.timestamp
    end

    add_index :forum_hyperlinks, :forum_id,              name: :index_forum_hyperlinks_on_forum_id

    create_table    :forum_languages do |t|
      t.belongs_to  :user
      t.string      :code,          limit:5
      t.string      :name,          limit:50
      t.string      :local_name,    limit:50
      t.string      :region,        limit:50
      t.boolean     :suggestion
      t.string      :state,         limit:50,         default:'suggesiton'
      t.boolean     :active,                          default:false
      t.timestamps
    end

    add_index :forum_languages, :user_id,               name: :index_forum_languages_on_user_id
    add_index :forum_languages, :code,                  name: :index_forum_languages_on_code
    add_index :forum_languages, :name,                  name: :index_forum_languages_on_name
    add_index :forum_languages, :region,                name: :index_forum_languages_on_region
    add_index :forum_languages, :active,                name: :index_forum_languages_on_active
    add_index :forum_languages, :suggestion,            name: :index_forum_languages_on_suggestion

    create_table    :forum_categories do |t|
      t.belongs_to  :user
      t.string      :name,           limit:50
      t.string      :state,          limit:50
      t.boolean     :suggestion
      t.boolean     :closed,                        default:false
      t.timestamps
    end
    add_index :forum_categories, :user_id,              name: :index_forum_categories_on_user_id
    add_index :forum_categories, :closed,               name: :index_forum_categories_on_closed
    add_index :forum_categories, :suggestion,           name: :index_forum_categories_on_suggestion

    create_table    :forum_photos do |t|
      t.belongs_to  :forum
      t.belongs_to  :user
      t.string      :name,            limit:50
      t.boolean     :resized,                      default:false
      t.string      :key
      t.timestamps
    end

    add_index :forum_photos, :user_id,                  name: :index_forum_photos_on_user_id
    add_index :forum_photos, :forum_id,                 name: :index_forum_photos_on_forum_id
    add_index :forum_photos, :name,                     name: :index_forum_photos_on_name
    add_index :forum_photos, :key,                      name: :index_forum_photos_on_key
    add_index :forum_photos, :resized,                  name: :index_forum_photos_on_resized

    create_table    :forum_topics do |t|
      t.belongs_to  :forum_category
      t.belongs_to  :user
      t.string      :name,              limit:50
      t.string      :state,             limit:20
      t.boolean     :closed,                        default:false
      t.boolean     :suggestion
      t.binary      :votes,                         default:[]
      t.boolean     :vote,                          default:false
      t.boolean     :restricted,                    default:false
      t.timestamps
    end

    add_index :forum_topics, :user_id,                  name: :index_forum_topics_on_user_id
    add_index :forum_topics, :forum_category_id,        name: :index_forum_topics_on_forum_category_id
    add_index :forum_topics, :closed,                   name: :index_forum_topics_on_closed
    add_index :forum_topics, :name,                     name: :index_forum_topics_on_name
    add_index :forum_topics, :vote,                     name: :index_forum_topics_on_vote
    add_index :forum_topics, :restricted,               name: :index_forum_topics_on_restricted
    add_index :forum_topics, :suggestion,               name: :index_forum_topics_on_suggestion

    create_table    :forum_people_lists do |t|
      t.belongs_to  :forum
      t.belongs_to  :forum_topic
      t.belongs_to  :forum_category
      t.string      :name,              limit:20
      t.string      :list,              limit:20
      t.binary      :people,                            default:[]
      t.timestamps
    end

    add_index :forum_people_lists, :forum_id,           name: :index_forum_people_lists_on_forum_id
    add_index :forum_people_lists, :forum_category_id,  name: :index_forum_people_lists_on_forum_category_id
    add_index :forum_people_lists, :forum_topic_id,     name: :index_forum_people_lists_on_forum_topic_id
    add_index :forum_people_lists, :name,               name: :index_forum_people_lists_on_name
    add_index :forum_people_lists, :list,               name: :index_forum_people_lists_on_list

    create_table    :forum_translations do |t|
      t.belongs_to  :forum
      t.belongs_to  :forum_topic
      t.belongs_to  :forum_category
      t.belongs_to  :forum_language
      t.belongs_to  :forum_hyperlink
      t.integer     :user_id
      t.boolean     :suggestion,                    default:false
      t.string      :state,         limit:20,       default:'new'
      t.text        :content
      t.text        :resolution
      t.timestamps
    end

    add_index :forum_translations, :forum_hyperlink_id,             name: :index_forum_translations_on_forum_hyperlink_id
    add_index :forum_translations, :forum_id,                     name: :index_forum_translations_on_forum_id
    add_index :forum_translations, :forum_topic_id,               name: :index_forum_translations_on_forum_topic_id
    add_index :forum_translations, :forum_category_id,            name: :index_forum_translations_on_forum_category_id
    add_index :forum_translations, :user_id,                      name: :index_forum_translations_on_user_id
    add_index :forum_translations, :forum_language_id,            name: :index_forum_translations_on_forum_language_id
    add_index :forum_translations, :state,                        name: :index_forum_translations_on_state


  end

end